# -*- coding: utf-8 -*-

__name__ = "gfzs"
__description__ = "Google Fuzzy Search"
__version__ = "0.2.1"
__copyright__ = "Copyright ©︎ 2021 yukihirop"
__url__ = "https://github.com/yukihirop/gfzs"
__author__ = "yukihirop"
__author_email__ = "te108186@gmail.com"
__license__ = "MIT"



import curses
import math
import os, sys

GOOGLE = "Google"
FUZZY = "Fuzzy"
SEARCH = "Search"

try:
    # need when 「python3 gfzs/views/header.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        from base import Base

        if os.environ.get("DEBUG"):
            import utils.debug as debug

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.views.base import Base

        if os.environ.get("DEBUG"):
            import gfzs.utils.debug as debug

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from views.base import Base

    if os.environ.get("DEBUG"):
        import utils.debug as debug


class Paging(Base):
    def __init__(self, stdscr, view):
        super().__init__(stdscr, None, "paging")
        self.view = view

    def create(self):
        self._init_layout()
        self._make_paging()
        self.window.refresh()

    def reset(self):
        self.destroy()
        self._init_layout()
        self._make_paging()
        self.window.refresh()

    def destroy(self):
        self.window.erase()

    def _init_layout(self):
        self.parent_height, self.parent_width = self.stdscr.getmaxyx()
        self.window = curses.newwin(2, self.parent_width, self.parent_height - 4, 0)

    # https://stackoverflow.com/a/53016371/9434894
    def _make_paging(self):
        begin_x = self.parent_width // 2 - 1
        per_page = self.view.per_page
        data_size = self.view.data_size
        paging = "{0}/{1}".format(
            self.view.current_page, math.ceil(data_size / per_page)
        )
        self.window.addstr(0, begin_x, paging, self.colors["common"] | curses.A_BOLD)


if __name__ == "__main__":

    class TestPaging(Paging):
        def run(self):
            self._loop()

        def _end_curses(self):
            """ Terminates the curses application. """
            curses.nocbreak()
            self.window.keypad(0)
            curses.echo()
            curses.endwin()

        def _loop(self):
            self.create()

            while True:
                try:
                    user_input = self.window.getch()
                except curses.error:
                    continue
                except KeyboardInterrupt:
                    break

                if user_input == curses.KEY_RESIZE:
                    self.reset()


if __name__ == "__main__":
    import os, sys
    import curses
    import signal
    import json

    # local

    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
    from model import Model
    from search_result import SearchResult
    from config.app import AppConfig

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    app_config = AppConfig.get_instance()
    if not app_config.valid():
        print("Config is invalid.")
        for error in app_config.errors:
            print("Error: %s" % error)
        sys.exit(1)

    json_str = open("fixtures/rust.json", "r").read()
    data = json.loads(json_str)

    # initscr() returns a window object representing the entire screen.
    stdscr = curses.initscr()

    # turn off automatic echoing of keys to the screen
    curses.noecho()
    # Buffering off
    # https://docs.python.org/ja/3/library/curses.html#curses.cbreak
    curses.cbreak()
    # Aable the mouse cursor.
    curses.curs_set(0)

    model = Model(data)
    model.update_query("")
    _ = model.find()

    view = SearchResult(stdscr, model)
    view.helper.current_selected = 1
    view.helper.per_page = 5

    target = TestPaging(stdscr, view)
    error = None
    try:
        target.run()
    except curses.error as e:
        error = str(e)
    finally:
        target._end_curses()
        if error != None:
            print(error)



import os
import sys

try:
    # need when 「python3 gfzs/views/footer.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        from utils import debug
        from utils.color import Color
        from config.app import AppConfig

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.utils.color import Color
        from gfzs.config.app import AppConfig

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from utils.color import Color
    from config.app import AppConfig


class Base(object):
    def __init__(self, stdscr, model, view_name):
        self.stdscr = stdscr
        self.parent_height, self.parent_width = stdscr.getmaxyx()
        self.model = model
        self.app_config = AppConfig.get_instance()
        self.color = Color.get_instance()
        self.color_data = self.app_config.data["view"][view_name]["color"]
        self.colors = self._create_colors(self.app_config, self.color_data)

    def _create_colors(self, app_config, color_data) -> dict:
        result = {}
        for view_name in color_data:
            result[view_name] = self.color.use(color_data[view_name])

        return result



import re
import os, sys

# local

try:
    # need when 「python3 gfzs/markup.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        import debug
        from multibyte import Multibyte
        from color import Color
        from config.app import AppConfig

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.utils.multibyte import Multibyte
        from gfzs.utils.color import Color
        from gfzs.config.app import AppConfig

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from utils.multibyte import Multibyte
    from utils.color import Color
    from config.app import AppConfig


class Markup:
    def __init__(self):
        self.multibyte = Multibyte()
        self.app_config = AppConfig.get_instance()
        self.color = Color.get_instance()
        self.color_data = self.app_config.data["view"]["search_result"]["color"]
        self.colors = self._create_colors(self.app_config, self.color_data)

    def parse(self, text, search_text):
        result = {}

        if search_text is None or search_text is "":
            return result

        remake_text = self.multibyte.marked_full_width(text)
        search_texts = [item for item in search_text.split(" ") if item]

        for st in search_texts:
            # Markup Partial
            result_at_partial = self._parse_as_partial(remake_text, st)
            for item in result_at_partial:
                result[item] = result_at_partial[item]

            # Markup Char
            result_at_char = self._parse_as_char(remake_text, st)
            for item in result_at_char:
                if not item in result:
                    result[item] = result_at_char[item]

        return result

    def _parse_as_partial(self, remake_text, search_text):
        result = {}
        remake_search_text = self.multibyte.marked_full_width(search_text)

        for m in re.finditer(remake_search_text, remake_text):
            if m is None:
                continue

            if not search_text in result:
                result[search_text] = []

            span = m.span()
            result[search_text].append(
                {
                    "half_width": {"start_index": span[0], "end_index": span[1]},
                    "color": self.colors["markup_partial"],
                    "match": self.multibyte.unmarked_full_width(m.group()),
                    "_type": "partial",
                }
            )

        return result

    def _parse_as_char(self, remake_text, search_text):
        result = {}
        char_pattern = re.compile(r"[{0}]".format(search_text))

        for m in char_pattern.finditer(remake_text):
            if m is None:
                continue

            if not search_text in result:
                result[search_text] = []

            span = m.span()
            result[search_text].append(
                {
                    "half_width": {"start_index": span[0], "end_index": span[1]},
                    "color": self.colors["markup_char"],
                    "match": m.group(),
                    "_type": "char",
                }
            )

        return result

    def _create_colors(self, app_config, color_data) -> dict:
        result = {}
        for view_name in color_data:
            result[view_name] = self.color.use(color_data[view_name])

        return result


if __name__ == "__main__":
    import curses

    # local

    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
    from config.app import AppConfig

    app_config = AppConfig.get_instance()
    if not app_config.valid():
        print("Config is invalid.")
        for error in app_config.errors:
            print("Error: %s" % error)
        sys.exit(1)

    try:
        # initscr() returns a window object representing the entire screen.
        stdscr = curses.initscr()

        markup = Markup()
        text = "Rustは非常に高速でメモリ効率が高くランタイムやガベージコレクタがないため、パフォーマンス重視のサービスを実装できますし、組込み機器上で実行したり他の言語との調和も簡単にできます。 信頼性. Rustの豊かな型システムと所有権 ..."

        search_text = "Rust 非常 効率"
        result = markup.parse(text, search_text)
        print("Partial: ", result)

        search_text = "パピプペポ"
        result = markup.parse(text, search_text)
        print("Char: ", result)

        search_text = None
        result = markup.parse(text, search_text)
        print("None: ", result)

        search_text = ""
        result = markup.parse(text, search_text)
        print("Blank: ", result)

        search_text = "\0"
        result = markup.parse(text, search_text)
        print("Null: ", result)

    finally:
        curses.endwin()



# coding:utf-8

import os
import sys
import json
import subprocess

# local

try:
    # need when 「python3 gfzs/cmd/config.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        import debug
        from config.app import AppConfig

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.config.app import AppConfig

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from config.app import AppConfig


def main():
    app_config = AppConfig.get_instance()
    config_path = app_config.config_path

    try:
        if os.path.exists(config_path):
            if os.environ["EDITOR"] != "":
                try:
                    subprocess.call([os.environ["EDITOR"], config_path])
                    sys.exit(0)
                except Exception:
                    raise Exception("Set the command to launch the editor in 'EDITOR'.")
            else:
                raise Exception("Set the environment variable 'EDITOR'")
        else:
            raise Exception("Config does not exist in %s" % config_path)
    except Exception as e:
        print("Error: %s" % str(e))
        sys.exit(1)


if __name__ == "__main__":
    main()



import pprint

pp = pprint.PrettyPrinter(indent=2)


def log(name, s=""):
    with open("./tmp/gfzs.log", "a") as f:
        f.write(str(name) + ":" + str(s) + "\n")


def dump(obj):
    with open("./tmp/gfzs.log", "a") as f:
        f.write(pp.pformat(obj) + "\n")

    return obj



# coding:utf-8

import json
import os, sys
import signal
import warnings
import curses

# local

try:
    # need when 「python3 gfzs/cmd/demo.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        from controller import Controller
        from config.app import AppConfig
        from config.runtime import RuntimeConfig

        if os.environ.get("DEBUG"):
            import debug

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.controller import Controller
        from gfzs.config.app import AppConfig
        from gfzs.config.runtime import RuntimeConfig

        if os.environ.get("DEBUG"):
            import gfzs.utils.debug as debug

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from controller import Controller
    from config.app import AppConfig
    from config.runtime import RuntimeConfig

    if os.environ.get("DEBUG"):
        import utils.debug as debug

RUST_JSON_DATA = [
    {
        "abstract": "Rustは非常に高速でメモリ効率が高くランタイムやガベージコレクタがないため、パフォーマンス重視のサービスを実装できますし、組込み機器上で実行したり他の言語との調和も簡単にできます。 信頼性. Rustの豊かな型システムと所有権 ...",
        "matches": [{"offset": 0, "phrase": "Rust"}, {"offset": 97, "phrase": "Rust"}],
        "title": "Rustプログラミング言語",
        "url": "https://www.rust-lang.org/ja",
    },
    {
        "abstract": "Rust（ラスト）はMozillaが支援するオープンソースのシステムプログラミング言語である。 Rust言語は速度、並行性、安全性を言語仕様として保証するC言語、C++に代わるシステムプログラミング（英語版）に適したプログラミング言語を ...",
        "matches": [{"offset": 0, "phrase": "Rust"}, {"offset": 48, "phrase": "Rust"}],
        "metadata": "型付け： 静的型付け、強い型付け、型推論、構造... | 登場時期： 2010年7月7日 | ライセンス： Apache-2.0、MIT License | 影響を受けた言語： Alef、C++、C Sharp、Cyclon...",
        "title": "Rust (プログラミング言語) - Wikipedia",
        "url": "https://ja.wikipedia.org/wiki/Rust_(%E3%83%97%E3%83%AD%E3%82%B0%E3%83%A9%E3%83%9F%E3%83%B3%E3%82%B0%E8%A8%80%E8%AA%9E)",
    },
    {
        "abstract": "趣味でRustを使ってコンパイラを作ってるらいパン粉です。 プログラミング言語Rustの良さを雑にあっぴるしていきます。 では、早速cargo run！(このコマンドでRustのプログラムが実行される) C,C++のコードが出てくる ...",
        "matches": [
            {"offset": 3, "phrase": "Rust"},
            {"offset": 39, "phrase": "Rust"},
            {"offset": 84, "phrase": "Rust"},
        ],
        "metadata": "2020/02/04",
        "title": "プログラミング言語Rustのススメ - Qiita",
        "url": "https://qiita.com/elipmoc101/items/3c8b6d8332a9019e578c",
    },
    {
        "abstract": "開発者のRusthへの愛情が、実世界での採用に反映されつつある。2020年6月2日、TIOBEは、初めてRustがTIOBE indexのトップ20に入ったことを発表した。",
        "matches": [{"offset": 52, "phrase": "Rust"}],
        "metadata": "2020/07/07",
        "title": "Rustが最も人気のあるプログラミング言語トップ20に - InfoQ",
        "url": "https://www.infoq.com/jp/news/2020/07/rust-top-20-language/",
    },
    {
        "abstract": "開発者向けQ＆Aサイト「Stack Overflow」は、先ごろ発表した年次開発者調査の結果を受け、プログラミング言語「Rust」が人気を集めているさまざまな理由について、ユーザーの生のコメントを紹介した。速度を犠牲にせず ...",
        "matches": [{"offset": 60, "phrase": "Rust"}],
        "metadata": "2020/06/11",
        "title": "「Rust」はなぜ人気があるのか、Stack Overflowがユーザーの ...",
        "url": "https://www.atmarkit.co.jp/ait/articles/2006/11/news051.html",
    },
    {
        "abstract": "Rustは「最も愛されている」プログラミング言語として不動の地位を誇っているものの、その普及は遅々として進んでいないようだ。",
        "matches": [{"offset": 0, "phrase": "Rust"}],
        "metadata": "2020/05/18",
        "title": "プログラミング言語「Rust」の普及に立ちはだかる壁 - ZDNet ...",
        "url": "https://japan.zdnet.com/article/35153014/",
    },
    {
        "abstract": "AWSはプログラミング言語「Rust」について、同社の長期的な戦略の重要に不可欠なコンポーネントだとしている。",
        "matches": [{"offset": 14, "phrase": "Rust"}],
        "metadata": "2020/11/30",
        "title": "AWS、プログラミング言語「Rust」を重視する理由示す ...",
        "url": "https://japan.zdnet.com/article/35163089/",
    },
    {
        "abstract": "AmazonでJim Blandy, Jason Orendorff, 中田 秀基のプログラミングRust。アマゾンならポイント還元本が多数。Jim Blandy, Jason Orendorff, 中田 秀基作品ほか、お急ぎ便対象商品は当日お届けも可能。またプログラミングRustもアマゾン配送 ...",
        "matches": [
            {"offset": 49, "phrase": "Rust"},
            {"offset": 137, "phrase": "Rust"},
        ],
        "title": "プログラミングRust | Jim Blandy, Jason Orendorff, 中田 秀基 ...",
        "url": "https://www.amazon.co.jp/%E3%83%97%E3%83%AD%E3%82%B0%E3%83%A9%E3%83%9F%E3%83%B3%E3%82%B0Rust-Jim-Blandy/dp/4873118557",
    },
    {
        "abstract": "オブジェクト指向と関数型の言語特徴を持ち、JavaやC#のようにVM（Virtual Machine）上の動作ではないため、既存のC言語のライブラリを直接扱える利点を持っています。またRustは、CやC++で発生しうる不正メモリアクセスに関する問題 ...",
        "matches": [{"offset": 92, "phrase": "Rust"}],
        "title": "プログラミング言語Rust入門 | 増田 智明 |本 | 通販 | Amazon",
        "url": "https://www.amazon.co.jp/%E3%83%97%E3%83%AD%E3%82%B0%E3%83%A9%E3%83%9F%E3%83%B3%E3%82%B0%E8%A8%80%E8%AA%9ERust%E5%85%A5%E9%96%80-%E5%A2%97%E7%94%B0-%E6%99%BA%E6%98%8E/dp/4822296857",
    },
    {
        "abstract": 'このテキストのこの版では、Rust 2018 Editionのイディオムを使うため、Rust 1.41.0かそれ以降を使っており、すべてのプロジェクトの Cargo.toml に edition="2018" とあることを前提にしています。 Rustをインストールしたりアップデート ...',
        "matches": [
            {"offset": 13, "phrase": "Rust"},
            {"offset": 42, "phrase": "Rust"},
            {"offset": 121, "phrase": "Rust"},
        ],
        "title": "The Rust Programming Language 日本語版 - The Rust ...",
        "url": "https://doc.rust-jp.rs/book-ja/",
    },
]


def main(args=None):
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    warnings.simplefilter("ignore", FutureWarning)

    _ = RuntimeConfig.get_instance(args)
    app_config = AppConfig.get_instance()
    if not app_config.valid():
        print("Config is invalid.")
        for error in app_config.errors:
            print("Error: %s" % error)
        sys.exit(1)

    data = RUST_JSON_DATA
    error = None
    controller = Controller(data)
    try:
        _ = controller.run()
    except curses.error as e:
        error = e
    except Exception as e:
        error = e
    finally:
        controller._end_curses()
        if error != None:
            print("Error: %s" % error)
            sys.exit(1)


if __name__ == "__main__":
    main()



import curses
import curses.ascii
import os, sys

# local

try:
    # need when 「python3 gfzs/views/footer.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        from utils import debug
        from utils.multibyte import Multibyte

        from base import Base

        if os.environ.get("DEBUG"):
            import utils.debug as debug
    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.utils.multibyte import Multibyte
        from gfzs.views.base import Base

        if os.environ.get("DEBUG"):
            import gfzs.utils.debug as debug
# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from utils.multibyte import Multibyte
    from views.base import Base

    if os.environ.get("DEBUG"):
        import utils.debug as debug

KEY_ENTER = 10
KEY_ESC = 27


class Footer(Base):
    def __init__(self, stdscr, model):
        super().__init__(stdscr, model, "footer")
        self.multibyte = Multibyte(stdscr)

    @property
    def query(self):
        return self.model.query

    @property
    def query_len(self):
        return self.multibyte.get_east_asian_width_count(self.query)

    @property
    def message(self):
        return self.app_config.data["view"]["footer"]["message"]

    @property
    def message_len(self):
        return self.multibyte.get_east_asian_width_count(self.message)

    def update_query(self, query):
        self.model.update_query(query)

    def create(self):
        self.update_query("")
        self._make_footer()

    def reset(self):
        self._make_footer()
        self.stdscr.move(
            self.parent_height - 1,
            self.message_len + 1 + self.query_len,
        )
        self.stdscr.refresh()

    def activate(self, is_init=False):
        # Able mouse cursor
        curses.curs_set(1)
        self.stdscr.move(
            self.parent_height - 1,
            self.message_len + 1 + self.query_len,
        )

        if is_init:
            self.stdscr.clrtoeol()

        self.stdscr.refresh()

    def delete_char(self):
        if self.query == None or self.query == "":
            self.stdscr.delch(self.parent_height - 1, self.message_len + 1 + 1)  # ?
            self.stdscr.delch(self.parent_height - 1, self.message_len + 1)  # ^
        else:
            query_len = self.query_len
            if query_len > 0:
                if self.multibyte.is_full_width(self.query[-1]):
                    k = 2
                else:
                    k = 1
                # backspace = ^?
                self.stdscr.delch(
                    self.parent_height - 1, self.message_len + 1 + query_len - k + 2
                )  # ?
                self.stdscr.delch(
                    self.parent_height - 1, self.message_len + 1 + query_len - k + 1
                )  # ^
                self.stdscr.delch(
                    self.parent_height - 1, self.message_len + 1 + query_len - k
                )
                self.update_query(self.query[:-1])

    def write(self, text):
        self.stdscr.addstr(text)
        self.model.push_query(text)

    # stdscr.getch doesn't work when I addstr to subwin
    def _make_footer(self):
        self.stdscr.hline(
            self.parent_height - 2,
            0,
            curses.ACS_HLINE | self.colors["hline"],
            self.parent_width,
        )
        self.stdscr.addstr(
            self.parent_height - 1, 0, self.message, self.colors["message"]
        )


if __name__ == "__main__":

    class TestFooter(Footer):
        def run(self):
            self.create()
            inp = self._loop()
            self._end_curses()
            return inp

        def _end_curses(self):
            """ Terminates the curses application. """
            curses.nocbreak()
            self.stdscr.keypad(0)
            curses.echo()
            curses.endwin()

        def _loop(self):
            self.activate(is_init=True)

            inp = ""
            while True:
                try:
                    user_input = self.multibyte.getch()
                except curses.error:
                    continue
                except KeyboardInterrupt:
                    break

                # Ah hitting enter, return the index of the selected list element.
                if user_input == curses.KEY_ENTER or user_input == KEY_ENTER:
                    return inp
                # https://www.programcreek.com/python/?code=mingrammer%2Fawesome-finder%2Fawesome-finder-master%2Fawesome%2Ftui.py
                elif user_input in (
                    curses.ascii.BS,
                    curses.ascii.DEL,
                    curses.KEY_BACKSPACE,
                ):
                    self.delete_char()
                    inp = inp[:-1]
                elif user_input == curses.KEY_RESIZE:
                    self.reset()
                else:
                    self.write(chr(user_input))
                    inp += chr(user_input)
                    self.update_query(inp)


if __name__ == "__main__":
    import os, sys
    import curses
    import signal

    # local

    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
    from model import Model
    from config.app import AppConfig

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    app_config = AppConfig.get_instance()
    if not app_config.valid():
        print("Config is invalid.")
        for error in app_config.errors:
            print("Error: %s" % error)
        sys.exit(1)

    # initscr() returns a window object representing the entire screen.
    stdscr = curses.initscr()

    # turn off automatic echoing of keys to the screen
    curses.noecho()
    # Buffering off
    # https://docs.python.org/ja/3/library/curses.html#curses.cbreak
    curses.cbreak()
    # Aable the mouse cursor.
    curses.curs_set(1)

    model = Model([])
    target = TestFooter(stdscr, model)
    error = None
    try:
        inp = target.run()
        print("query:", model.query)
        print("result:", inp)
    except curses.error as e:
        error = str(e)
    finally:
        if error != None:
            print(error)



# ref: https://qiita.com/ttsubo/items/c4af71ceba15b5b213f8

import argparse

DEFAULT_SCORE = 30


class Singleton(object):
    @classmethod
    def get_instance(cls, args=None):
        if not hasattr(cls, "_instance"):
            cls._instance = cls(args)
        else:
            if args != None:
                cls._instance.args = args

        return cls._instance


class RuntimeConfig(Singleton):
    """A class that reads and manages the options passed in the runtime"""

    """fuzzywuzzy's score. please see https://github.com/seatgeek/fuzzywuzzy"""
    default_score = DEFAULT_SCORE

    def __init__(self, args):
        self.args = args

    @property
    def score(self):
        return self.args.score or RuntimeConfig.default_score



import curses
import os, sys

# local

try:
    # need when 「python3 gfzs/markup.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        import debug

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug


class Singleton(object):
    @classmethod
    def get_instance(cls):
        if not hasattr(cls, "_instance"):
            cls._instance = cls()
        return cls._instance


class Color(Singleton):
    COLORS = (
        curses.COLOR_BLACK,
        curses.COLOR_RED,
        curses.COLOR_GREEN,
        curses.COLOR_YELLOW,
        curses.COLOR_BLUE,
        curses.COLOR_MAGENTA,
        curses.COLOR_CYAN,
        curses.COLOR_WHITE,
    )

    def __init__(self):
        curses.start_color()
        curses.use_default_colors()
        colors_len = len(Color.COLORS)
        self.offset_after_builtin_color = self._calculate_color_number(
            colors_len, colors_len
        )
        self._setup()

    def _setup(self):
        for i in Color.COLORS:
            for j in Color.COLORS:
                color_number = self._calculate_color_number(i, j)
                curses.init_pair(color_number, i, j)

        # https://www.it-swarm-ja.tech/ja/python/curses%E3%81%A7%E7%AB%AF%E6%9C%AB%E3%81%AE%E3%82%AB%E3%83%A9%E3%83%BC%E3%83%91%E3%83%AC%E3%83%83%E3%83%88%E3%82%92%E4%BD%BF%E7%94%A8%E3%81%99%E3%82%8B%E6%96%B9%E6%B3%95/1042208550/
        # The default color is assigned to the color number -1.
        offset = self.offset_after_builtin_color
        curses.init_pair(28 + offset, 27, -1)  # G(g) color of Google
        curses.init_pair(10 + offset, 9, -1)  # o(e) color of Google
        curses.init_pair(12 + offset, 11, -1)  # o color of Google
        curses.init_pair(36 + offset, 35, -1)  # l color of Google

    def _end_curses(self):
        """ Terminates the curses application. """
        curses.nocbreak()
        curses.echo()
        curses.endwin()

    def use(self, data) -> int:
        i = data["text"]
        j = data["background"]
        style = data["style"]

        try:
            if j == -1:
                offset = self.offset_after_builtin_color
                color_number = i + 1 + offset
            else:
                color_number = self._calculate_color_number(i, j)

            if style == "normal":
                return curses.color_pair(color_number)
            elif style == "bold":
                return curses.color_pair(color_number) | curses.A_BOLD
            elif style == "link":
                return curses.color_pair(color_number) | curses.A_UNDERLINE
            else:
                raise Exception("[Color] Do not support style: %s" % style)
        except Exception as e:
            self._end_curses()
            print("Error: %s" % str(e))
            sys.exit(1)

    def google(self, c, style="bold") -> int:
        offset = self.offset_after_builtin_color
        color = None

        try:
            if c in ("G", "g"):
                color = curses.color_pair(28 + offset)
            elif c == "o":
                color = curses.color_pair(10 + offset)
            elif c == "o2":
                color = curses.color_pair(12 + offset)
            elif c == "l":
                color = curses.color_pair(36 + offset)
            elif c == "e":
                color = curses.color_pair(10 + offset)

            if style == "bold":
                return color | curses.A_BOLD
            elif style == "normal":
                return color
            else:
                raise Exception("[Color] Do not support style: %s" % style)
        except Exception as e:
            self._end_curses()
            print("Error: %s" % str(e))
            sys.exit(1)

    @property
    def highlight(self) -> int:
        data = {
            "text": curses.COLOR_BLACK,
            "background": curses.COLOR_GREEN,
            "style": "normal",
        }
        return self.use(data)

    def fuzzy(self, style="bold") -> int:
        return self.google("o2", style)

    def search(self, style="bold") -> int:
        return self.google("o", style)

    def version(self, style="bold") -> int:
        data = {
            "text": curses.COLOR_GREEN,
            "background": curses.COLOR_BLACK,
            "style": style,
        }
        return self.use(data)

    def copy_right(self, style="bold") -> int:
        data = {
            "text": curses.COLOR_GREEN,
            "background": curses.COLOR_BLACK,
            "style": style,
        }
        return self.use(data)

    def not_found(self, style="bold") -> int:
        data = {
            "text": curses.COLOR_GREEN,
            "background": curses.COLOR_BLACK,
            "style": style,
        }
        return self.use(data)

    # https://www.linuxjournal.com/content/about-ncurses-colors-0
    def _calculate_color_number(self, fg, bg):
        B = 1 << 7
        bbb = (7 & bg) << 4
        ffff = 7 & fg
        return B | bbb | ffff


if __name__ == "__main__":

    import curses
    import signal

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    def demo(screen):
        color = Color.get_instance()

        for i in range(0, 8):
            for j in range(0, 8):
                use_color = color.use({"text": i, "background": j, "style": "normal"})
                screen.addstr("(%d, %d)" % (i, j), use_color)

        screen.getch()

    curses.wrapper(demo)



# coding:utf-8

import os
import sys
import json

# local

try:
    # need when 「python3 gfzs/cmd/init.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        import debug
        from config.app import AppConfig

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.utils import debug
        from gfzs.config.app import AppConfig

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from utils import debug
    from config.app import AppConfig


def main():
    app_config = AppConfig.get_instance()
    config_path = app_config.config_path
    data = app_config.data

    if os.path.exists(config_path):
        print("Reinitialized existing in %s" % config_path)
    else:
        with open(config_path, mode="w") as f:
            f.write(json.dumps(data, indent=2))
            print("Initialize config in %s" % config_path)

    sys.exit(0)


if __name__ == "__main__":
    main()



import sys
import os
import signal
import warnings
import webbrowser
import json
import curses
import argparse

# local

try:
    # need when 「cat fixtures/rust.json | bin/gfzs」
    from gfzs import tty, info
    from gfzs.utils import debug
    from gfzs.controller import Controller
    from gfzs.model import Model
    from gfzs.config.runtime import RuntimeConfig
    from gfzs.config.app import AppConfig
    import gfzs.cmd.init as cmd_init
    import gfzs.cmd.edit as cmd_edit
    import gfzs.cmd.demo as cmd_demo
    import gfzs.cmd.valid as cmd_valid

# need when 「cat fixtures/rust.json | python -m gfzs」
except ModuleNotFoundError:
    from utils import debug
    from controller import Controller
    from model import Model
    from config.runtime import RuntimeConfig
    from config.app import AppConfig
    import cmd.init as cmd_init
    import cmd.edit as cmd_edit
    import cmd.demo as cmd_demo
    import cmd.valid as cmd_valid


def open_tty(ttyname):
    # See https://github.com/stefanholek/term/issues/1
    return open(ttyname, "wb+", buffering=0)


def init_parser():
    parser = argparse.ArgumentParser(
        prog="gfzs",
        description="Google Fuzzy Search. Pipe the search result(json) of googler and use it",
    )

    parser.add_argument("--version", "-v", action="version", version=info.__version__)
    parser.add_argument(
        "--score",
        "-s",
        type=int,
        default=RuntimeConfig.default_score,
        help="fuzzywuzzy's score. please see https://github.com/seatgeek/fuzzywuzzy",
    )

    subparsers = parser.add_subparsers(title="SubCommands", dest="command")
    subparsers.required = False

    subparsers.add_parser("init", help="Initialize gfzs")
    subparsers.add_parser("edit", help="Edit config")
    subparsers.add_parser("demo", help="Play with Demo")
    subparsers.add_parser("valid", help="Validate ~/.gfzsrc")

    return parser


def exec_subcommand(parser, argv=sys.argv[1:]) -> None:
    args = parser.parse_args(argv)

    if args.command == "init":
        cmd_init.main()
    elif args.command == "edit":
        cmd_edit.main()
    elif args.command == "demo":
        cmd_demo.main(args)
    elif args.command == "valid":
        cmd_valid.main()


def main() -> None:

    signal.signal(signal.SIGINT, signal.SIG_DFL)
    # https://note.nkmk.me/python-warnings-ignore-warning/
    warnings.simplefilter("ignore", FutureWarning)

    parser = init_parser()
    exec_subcommand(parser)

    data = None
    errors = []
    printable_len = 100

    args = parser.parse_args()
    _ = RuntimeConfig.get_instance(args)
    ttyname = tty.get_ttyname()

    with open_tty(ttyname) as tty_f:
        _ = tty.reconnect_descriptors(tty_f)

        try:
            json_str = sys.stdin.read()
            data = json.loads(json_str)
            app_config = AppConfig.get_instance()
            validator = Model(data)

            if not app_config.valid():
                print("Config is invalid.")
                errors = app_config.errors
                return
            elif not validator.valid():
                errors = validator.errors
                return
        except json.decoder.JSONDecodeError as e:
            print("Error: %s" % e)
            if "[ERROR]" in json_str or len(json_str) <= printable_len:
                print("Input data: %s" % json_str)
            else:
                print("Input data (100 chars): %s ..." % json_str[:printable_len])

            sys.exit(1)
        except Exception as e:
            errors.append(e)
        finally:
            if errors != []:
                for error in errors:
                    print("Error: %s" % error)

                sys.exit(1)

        controller = Controller(data)
        try:
            _ = controller.run()
        except curses.error as e:
            error = e
        except Exception as e:
            error = e
        finally:
            controller._end_curses()
            if error != None:
                print(error)
                sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gfzs', 'gfzs.cmd', 'gfzs.config', 'gfzs.utils', 'gfzs.views']

package_data = \
{'': ['*']}

install_requires = \
['flatten-dict>=0.3.0,<0.4.0',
 'fuzzywuzzy>=0.18.0,<0.19.0',
 'python-Levenshtein>=0.12.1,<0.13.0']

entry_points = \
{'console_scripts': ['gfzs = gfzs.cli:main']}

setup_kwargs = {
    'name': 'gfzs',
    'version': '0.2.1',
    'description': 'Google Fuzzy Search',
    'long_description': '![gfzs](https://user-images.githubusercontent.com/11146767/105204059-25556c80-5b87-11eb-862e-828a80aa6a94.png)\n\n`Google Fuzzy Search` is a tool for fuzzy find for the result of searching with [googler](https://github.com/jarun/googler).\n\n![gfzs](https://user-images.githubusercontent.com/11146767/105366627-e2f16580-5c42-11eb-92ef-8cd1aa87ce77.gif)\n\n![image](https://user-images.githubusercontent.com/11146767/105366973-47142980-5c43-11eb-89d1-9e24e6a48106.png)\n\n## ⚠ Warning\n\n![alert_tape](https://user-images.githubusercontent.com/11146767/105363618-93f60100-5c3f-11eb-8cb9-4f01ec49b486.png)\n\nIf you get the following response, stop using it.\nThis is an error from `google`. So [you can\'t do anything with googler](https://github.com/jarun/googler/issues/276#issuecomment-482754595). Please use it after a while.\n\n```\n$ googler --json --count 100 python | gfzs\nError: Expecting value: line 1 column 2 (char 1)\nInput data: [ERROR] Connection blocked due to unusual activity. THIS IS NOT A BUG, please do NOT report it as a bug unless you have specific information that may lead to the development of a workaround. You IP address is temporarily or permanently blocked by Google and requires reCAPTCHA-solving to use the service, which googler is not capable of. Possible causes include issuing too many queries in a short time frame, or operating from a shared / low reputation IP with a history of abuse. Please do NOT use googler for automated scraping.\n```\n\nTo avoid such problems, __it is better not to set `googler\'s count option`.__\n\n__We are not responsible if you ignore the advice and block your IP. please note that.__\n\n![alert_tape](https://user-images.githubusercontent.com/11146767/105363618-93f60100-5c3f-11eb-8cb9-4f01ec49b486.png)\n\n## 🐍 Install\n\n```bash\npip install gfzs\n```\n\n## 😎 Demo\n\nseeing is believing.  \nYou can easily try what kind of app it is.\n\n```bash\ngfzs demo\n```\n\n## 🌍 Environment\n\n- python3 (over Python 3.6.1)\n- poetry\n\n## 📖 Usage\n\nInitialize first. A configuration file (`.gfzsrc`) is created in your home directory.\n\n```bash\n$ gfzs init\nInitialize config in /Users/yukihirop/.gfzsrc\n```\n\nNext, Prepare json with `title`, `url` and `abstract` as keys and pass it.\nFor Example, Assuming that data.json is as follows.\n\n`data.json`\n\n<details>\n\n```json\n[\n  {\n    "abstract": "Rustは非常に高速でメモリ効率が高くランタイムやガベージコレクタがないため、パフォーマンス重視のサービスを実装できますし、組込み機器上で実行したり他の言語との調和も簡単にできます。 信頼性. Rustの豊かな型システムと所有権 ...",\n    "matches": [\n      {\n        "offset": 0,\n        "phrase": "Rust"\n      },\n      {\n        "offset": 97,\n        "phrase": "Rust"\n      }\n    ],\n    "title": "Rustプログラミング言語",\n    "url": "https://www.rust-lang.org/ja"\n  },\n  {\n    "abstract": "Rust（ラスト）はMozillaが支援するオープンソースのシステムプログラミング言語である。 Rust言語は速度、並行性、安全性を言語仕様として保証するC言語、C++に代わるシステムプログラミング（英語版）に適したプログラミング言語を ...",\n    "matches": [\n      {\n        "offset": 0,\n        "phrase": "Rust"\n      },\n      {\n        "offset": 48,\n        "phrase": "Rust"\n      }\n    ],\n    "metadata": "型付け： 静的型付け、強い型付け、型推論、構造... | 登場時期： 2010年7月7日 | ライセンス： Apache-2.0、MIT License | 影響を受けた言語： Alef、C++、C Sharp、Cyclon...",\n    "title": "Rust (プログラミング言語) - Wikipedia",\n    "url": "https://ja.wikipedia.org/wiki/Rust_(%E3%83%97%E3%83%AD%E3%82%B0%E3%83%A9%E3%83%9F%E3%83%B3%E3%82%B0%E8%A8%80%E8%AA%9E)"\n  }\n]\n```\n\n</details>\n\nPipe as standard input\n\n```bash\ncat data.json | gfzs\n```\n\n## 🚀 Practical Example (using googler)\n\n__It is recommended to use [alias](https://github.com/yukihirop/gfzs/wiki/%F0%9F%8D%96-Tips) to avoid accidentally passing the -n (--count) option to googler.__\n\n#### Search python articles on github.com\n\n```bash\ngoogler --json --site github.com python | gfzs\n```\n\n#### Display only those with a score of 50 or more in the search results\n\nPlease see here for [score](https://github.com/seatgeek/fuzzywuzzy).\n\n```bash\ngoogler --json --site github.com python | gfzs -s 50\n```\n\n## ⚙ Configure\n\nThe default setting is the following file, and each key can enter the following values.  \n\nAfter changing the settings, you can check if the settings are correct with the `valid` command. \n\n```bash\n$ gfzs valid\nConfig is valid\n```\n\n`~/.gfzsrc`\n\n```json\n{\n  "view": {\n    "footer": {\n      "message": "QUERY>",\n      "color": {\n        "message": {\n          "text": 2,\n          "background": 0,\n          "style": "normal"\n        },\n        "hline": {\n          "text": 7,\n          "background": 0,\n          "style": "normal"\n        }\n      }\n    },\n    "header": {\n      "color": {\n        "hline": {\n          "text": 7,\n          "background": 0,\n          "style": "normal"\n        }\n      }\n    },\n    "search_result": {\n      "color": {\n        "index": {\n          "text": 6,\n          "background": 0,\n          "style": "normal"\n        },\n        "title": {\n          "text": 2,\n          "background": 0,\n          "style": "bold"\n        },\n        "url": {\n          "text": 3,\n          "background": 0,\n          "style": "link"\n        },\n        "abstract": {\n          "text": 7,\n          "background": 0,\n          "style": "normal"\n        },\n        "markup_partial": {\n          "text": 2,\n          "background": 5,\n          "style": "normal"\n        },\n        "markup_char": {\n          "text": 1,\n          "background": 0,\n          "style": "normal"\n        }\n      }\n    },\n    "paging": {\n      "color": {\n        "common": {\n          "text": 2,\n          "background": 0,\n          "style": "bold"\n        }\n      }\n    }\n  }\n}\n```\n\n|key|description|value|\n|---|-----------|-----|\n|<kbd>text</kbd>|curses color|`0〜7`|\n|<kbd>background</kbd>|curses color|`0〜7`|\n|<kbd>style</kbd>|text style|`"normal", "link", "bold"`|\n\n### curses color\n\n|number|description|\n|------|-----------|\n|`0`|`curses.COLOR_BLACK`|\n|`1`|`curses.COLOR_RED`|\n|`2`|`curses.COLOR_GREEN`|\n|`3`|`curses.COLOR_YELLOW`|\n|`4`|`curses.COLOR_BLUE`|\n|`5`|`curses.COLOR_MAGENTA`|\n|`6`|`curses.COLOR_CYAN`|\n|`7`|`curses.COLOR_WHITE`|\n\n## ⚙ Environment Variable\n\n|name|description|\n|----|-----------|\n|`DEBUG`|You will be able to use the `debug` module.|\n|`EDITOR`|Set the command to open the editor.|\n\n## 💪 Development\n\nFirst, create a virtual environment.\n\n```bash\n$ python3 -m venv .venv\n$ poetry shell\n$ poetry install\n```\n\n```bash\n$ python3 gfzs/controller.py\n```\n\nor\n\n```bash\n$ cat fixtures/rust.json | python3 -m gfzs\n```\n\nor\n\n```bash\n$ cat fixtures/rust.json | bin/gfzs\n```\n\n## 💪 Development (Check Partial View)\n\n### search_result\n\n```bash\n$ python3 gfzs/views/search_result.py\n```\n\n![image](https://user-images.githubusercontent.com/11146767/104927269-8e50af00-59e4-11eb-8fcf-8b18f4db4420.png)\n\n### footer\n\n```bash\n$ python3 gfzs/views/footer.py\n```\n\n![image](https://user-images.githubusercontent.com/11146767/104927398-b93b0300-59e4-11eb-99a8-5b1e85dc000d.png)\n\n### header\n\n```bash\n$ python3 gfzs/views/header.py\n```\n\n![image](https://user-images.githubusercontent.com/11146767/104927550-ed162880-59e4-11eb-8d5c-efca84c58ea6.png)\n\n### not_found\n\n```bash\n$ python3 gfzs/views/not_found.py\n```\n\n![image](https://user-images.githubusercontent.com/11146767/104927669-0d45e780-59e5-11eb-9dc8-76baec0c04ac.png)\n\n\n### paging\n\n```bash\n$ python3 gfzs/views/paging.py\n```\n\n![image](https://user-images.githubusercontent.com/11146767/104927802-36667800-59e5-11eb-9afb-2b3386c4d8c6.png)\n\n\n## 📚 Reference\n\nI really referred to the implementation of the following repository.\n\n- [mooz/percol](https://github.com/mooz/percol)\n- [mingrammer/awesome-finder](https://github.com/mingrammer/awesome-finder)\n- [frankdice/fuzzyui](https://github.com/frankdice/fuzzyui)\n- [NikolaiT/Scripts](https://github.com/NikolaiT/Scripts/blob/master/scripts/python/curses/text_selector.py)\n',
    'author': 'yukihirop',
    'author_email': 'te108186@gmail.com',
    'maintainer': 'yukihirop',
    'maintainer_email': 'te108186@gmail.com',
    'url': 'https://github.com/yukihirop/gfzs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# coding:utf-8

import json
import os, sys
import signal
import warnings

# local

try:
    # need when 「python3 gfzs/cmd/valid.py」
    if __name__ == "__main__":
        # https://codechacha.com/ja/how-to-import-python-files/
        sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
        from config.app import AppConfig

        if os.environ.get("DEBUG"):
            import debug

    # need when 「cat fixtures/rust.json | python -m gfzs」
    # need when 「cat fixtures/rust.json | bin/gfzs」
    else:
        from gfzs.config.app import AppConfig

        if os.environ.get("DEBUG"):
            import gfzs.utils.debug as debug

# need when 「python3 gfzs/controller.py」
except ModuleNotFoundError:
    # https://codechacha.com/ja/how-to-import-python-files/
    sys.path.append(os.path.dirname(os.path.abspath(os.path.dirname("../"))))
    from config.app import AppConfig

    if os.environ.get("DEBUG"):
        import utils.debug as debug


def main():
    app_config = AppConfig.get_instance()
    if app_config.valid():
        print("Config is valid.")
    else:
        print("Config is invalid.")
        for error in app_config.errors:
            print("Error: %s" % error)

    sys.exit(0)


if __name__ == "__main__":
    main()
